package com.diagbot.process;

import com.diagbot.biz.push.entity.Lis;
import com.diagbot.biz.push.entity.Pacs;
import com.diagbot.dto.BillMsg;
import com.diagbot.dto.BillNeoDTO;
import com.diagbot.dto.BillNeoMaxDTO;
import com.diagbot.dto.IndicationDTO;
import com.diagbot.dto.NodeNeoDTO;
import com.diagbot.dto.WordCrfDTO;
import com.diagbot.enums.NeoEnum;
import com.diagbot.model.entity.*;
import com.diagbot.model.label.ChiefLabel;
import com.diagbot.model.label.DiagLabel;
import com.diagbot.model.label.PastLabel;
import com.diagbot.model.label.PresentLabel;
import com.diagbot.rule.AgeRule;
import com.diagbot.rule.DiagRule;
import com.diagbot.rule.LisRule;
import com.diagbot.rule.PacsRule;
import com.diagbot.rule.SexRule;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.ListUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

/**
 * @Description: 开单总入口
 * @author: zhoutg
 * @time: 2018/8/6 9:11
 */
@Component
public class BillProcess {

    public void process(List<BillNeoDTO> billNeoDTOList, WordCrfDTO wordCrfDTO, IndicationDTO res) {
        // 合并图谱数据到同一个对象中
        List<BillNeoMaxDTO> billNeoMaxDTOList = new ArrayList<>();
        for (BillNeoDTO billNeoDTO : billNeoDTOList) {
            BillNeoMaxDTO billNeoMaxDTO =  new BillNeoMaxDTO();
            if (billNeoDTO.getDrugBillNeoDTO() != null) {
                BeanUtil.copyProperties(billNeoDTO.getDrugBillNeoDTO(), billNeoMaxDTO);
            } else if (billNeoDTO.getLisBillNeoDTO() != null) {
                BeanUtil.copyProperties(billNeoDTO.getLisBillNeoDTO(), billNeoMaxDTO);
            } else if (billNeoDTO.getPacsBillNeoDTO() != null) {
                BeanUtil.copyProperties(billNeoDTO.getPacsBillNeoDTO(), billNeoMaxDTO);
            }
            billNeoMaxDTO.setOrderName(billNeoDTO.getName()); // 开单名称
            billNeoMaxDTO.setOrderStandName(billNeoDTO.getStandname()); // 开单标准名称
            // 测试数据开始
            if (billNeoDTO.getName().equals("普通胃镜检查")) {
                NodeNeoDTO sexNeo = new NodeNeoDTO();
                sexNeo.setName("男");
                billNeoMaxDTO.setGender(sexNeo);

                NodeNeoDTO nodeNeoDTO = new NodeNeoDTO();
                nodeNeoDTO.setName("红细胞压积");
                nodeNeoDTO.setVal("阳性");
                billNeoDTO.getPacsBillNeoDTO().getLis().add(nodeNeoDTO);

                NodeNeoDTO opereat = new NodeNeoDTO();
                opereat.setName("胸腔镜下左下肺叶切除术");
                billNeoDTO.getPacsBillNeoDTO().getOperations().add(opereat);
                NodeNeoDTO opereat2 = new NodeNeoDTO();
                opereat2.setName("区域淋巴结清扫术");
                billNeoDTO.getPacsBillNeoDTO().getOperations().add(opereat2);

                NodeNeoDTO drug = new NodeNeoDTO();
                drug.setName("拜阿司匹林");
                billNeoDTO.getPacsBillNeoDTO().getAllergicmeds().add(drug);
                NodeNeoDTO drug2 = new NodeNeoDTO();
                drug2.setName("美多芭");
                billNeoDTO.getPacsBillNeoDTO().getAllergicmeds().add(drug2);

                NodeNeoDTO drug3 = new NodeNeoDTO();
                drug3.setName("泰舒达");
                billNeoDTO.getPacsBillNeoDTO().getOralmeds().add(drug3);

            }
            // 测试数据结束

            System.out.println(billNeoMaxDTO);
            billNeoMaxDTOList.add(billNeoMaxDTO);
        }
        processRule(billNeoMaxDTOList, wordCrfDTO, res);
    }

    // 规则处理
    public void processRule(List<BillNeoMaxDTO> billNeoMaxDTOList, WordCrfDTO wordCrfDTO, IndicationDTO indicationDTO) {
        DiagLabel diagLabel = wordCrfDTO.getDiagLabel();
        ChiefLabel chiefLabel = wordCrfDTO.getChiefLabel();
        PresentLabel presentLabel = wordCrfDTO.getPresentLabel();
        PastLabel pastLabel = wordCrfDTO.getPastLabel();
        List<Diag> diags = new ArrayList<>();
        List<Clinical> clinicals = new ArrayList<>();
        List<Operation> operations_present = presentLabel.getOperations();
        List<Operation> operations_past = pastLabel.getOperations();
        operations_present.addAll(operations_past);
        if(diagLabel != null){
            diags = diagLabel.getDiags();
        }
        if(chiefLabel != null && ListUtil.isNotEmpty(chiefLabel.getClinicals())){
            clinicals.addAll(chiefLabel.getClinicals());
        }
        // 现病史临床表现数据先不用
        if(presentLabel != null && ListUtil.isNotEmpty(presentLabel.getClinicals())){
            clinicals.addAll(presentLabel.getClinicals());
        }
        List<Lis> lis = wordCrfDTO.getLis();
        List<Pacs> pacs = wordCrfDTO.getPacs();

        List<BillMsg> billMsgList = new ArrayList<>();
        for (BillNeoMaxDTO bill : billNeoMaxDTOList) {
            // 性别
            SexRule.compareSexWithBill(wordCrfDTO, bill, billMsgList, NeoEnum.gender.getName());

            // 年龄
            AgeRule.compareAgeWithBill(wordCrfDTO, bill, billMsgList, NeoEnum.ageNeoDTO.getName());

            // 诊断
            DiagRule.compareDiseaseWithBill(bill.getDisease(), diags, bill, billMsgList, NeoEnum.disease.getName());

            //化验
            LisRule.compareLisWithBill(lis, bill, billMsgList, NeoEnum.lis.getName());

             //辅检
            PacsRule.comparePacsWithBill(bill.getPacs(), pacs, bill, billMsgList, NeoEnum.pacs.getName());

            //临床表现
            DiagRule.compareDiseaseWithBill(bill.getClinicfindings(), clinicals, bill, billMsgList, NeoEnum.clinicfindings.getName());

            //手术(既往史、现病史)
            DiagRule.compareDiseaseWithBill(bill.getOperations(), operations_present, bill, billMsgList, NeoEnum.operations.getName());

            //禁忌过敏药品（既往史）
            DiagRule.compareDiseaseWithBill(bill.getAllergicmeds(), pastLabel.getAllergyMedicines(), bill, billMsgList, NeoEnum.allergicmeds.getName());

            //服用药品（现病史一般情况后的药品）
            DiagRule.compareDiseaseWithBill(bill.getOralmeds(), takeMedicine(presentLabel), bill, billMsgList, NeoEnum.oralmeds.getName());

        }
        indicationDTO.setBillMsgList(billMsgList);
    }

    /**
     * 一般情况以后的药品
     * @param presentLabel
     */
    public List<Medicine> takeMedicine(PresentLabel presentLabel){
        List<Medicine> takems = new ArrayList<>();
        List<GeneralDesc> generals = presentLabel.getGenerals();
        List<Medicine> medicines = presentLabel.getMedicines();
        if (generals.size() > 0) {
            String presentText = presentLabel.getText();
            if (StringUtils.isNotBlank(presentText) && medicines != null && medicines.size() > 0) {
                String lastGeneral = generals.get(generals.size() - 1).getName();
                int lastGeneralIndex = presentText.lastIndexOf(lastGeneral);
                for (Medicine medicine : medicines) {
                    /* 现病史中一般情况之后的药品名称,并且不包含不详 */
                    int medicine_index = presentText.indexOf(medicine.getName());
                    if (medicine_index > lastGeneralIndex && !medicine.getName().contains("不详")) {
                        takems.add(medicine);
                    }
                }
            }
        }
        return takems;

    }
}
